--[[
************************************************************************
BuddyName.lua
************************************************************************
This addon is released under GPL v3. All additionally used libraries are
released under the license that accompanies them.
]]

BuddyName = LibStub("AceAddon-3.0"):NewAddon("BuddyName",
                                             "AceConsole-3.0")

-- Chat events that will be processed
local chatEvents = {
	 ["CHAT_MSG_CHANNEL"]        = false, -- take out?
	 ["CHAT_MSG_SAY"]            = true,
	 ["CHAT_MSG_YELL"]           = true,
	 ["CHAT_MSG_WHISPER"]        = true,
	 ["CHAT_MSG_GUILD"]          = true,
	 ["CHAT_MSG_OFFICER"]        = true,
	 ["CHAT_MSG_RAID"]           = true,
	 ["CHAT_MSG_RAID_LEADER"]    = true,
}

local slashCommands = {
	 ["bn"]        = true,
	 ["buddyname"] = true,
	 ["BuddyName"] = true,
}

--[[ Local variables ]]
local BN_DB_Version = 1; -- Database format
local BN_CommVersion = 1; -- Message format communication protocol
local BN_Activated = false; -- Is this addon active? Modified by OnEnable and OnDisable

--[[ Transforms old DB entries to newer versions ]]
function BuddyName:UpdateDB()
	 if (self.db.realm.version==1) then
			self.db.realm.version=2
			-- perform update to v2 here
	 end

	 --[[
	 if (self.db.realm.version==2) then
			self.db.realm.version=3
			-- perform update to v3 here
	 end
	 ]]

end

--[[ Checks whether the given name is a valid player and has a buddy name in our DB ]]
function BuddyName:IsInDB(author)
	 if (author~=nil and author~="") then
			-- if (UnitExists(author) and UnitIsPlayer(author)) then -- TODO: UnitExists returns only true if unit exists and is in current zone => remove
				 if (self.db.realm.char and self.db.realm.char[author] and self.db.realm.char[author]~="") then
						return true,self.db.realm.char[author];
				 end
			-- end
	 end
	 return false, nil
end

--[[ Message filter, will modify author information. If returns true, message is discarded ]]
local function ChatFilter(chatFrame, event, msg, author, ...)
	 if (chatEvents[event]==true) then -- really one of the filtered events (just in case)
			local replace,buddy = BuddyName:IsInDB(author) -- char in DB?
			if (replace) then
				 author=string.format("%s:%s", author, buddy)
			end
	 end
	 
	 return false, msg, author, ...
end

--[[ Process slash commands (unexpected, eh?) ]]
function BuddyName:SlashProcessor(input)
	 -- 'input' contains whatever follows the slash command

	 -- split input string, ignore additional whitespace
	 local tbl = {}
	 for v in string.gmatch(input, "[^ ]+") do
			tinsert(tbl, v)
	 end

	 -- if no additional parameters => print out help text
	 if (#tbl == 0) then
			self:Print("Some help text should appear here");
			return;
	 end

	 local command = tremove(tbl,1)

	 -- Activate additional functionality
	 if (command=="on") then
			self:Enable()
			self:Print("Activated");
			return
	 end

	 -- Disable additional functionality
	 if (command=="off") then
			self:Disable()
			self:Print("Deactivated");
			return
	 end

	 if (not BN_Active) then
			self:Print("Not active at the moment")
			return
	 end

	 -- add another person to the list
	 if (command=="add") then
			if (#tbl~=2) then
				 self:Print("Add: Wrong number of parameters")
			end
			-- TODO: Check same things as in chatfilter: Is char really player, etc
			BuddyName.db.realm.char[tbl[1]]=tbl[2]
			self:Print(string.format("Added buddy name: %s => %s", tbl[1], tbl[2]));
			return
	 end

	 -- add another person to the list
	 if (command=="del" or command=="rm") then
			if (#tbl~=1) then
				 self:Print("Del: Wrong number of parameters")
			end
			if (not BuddyName.db.realm.char[tbl[1]]) then
				 self:Print("Del: Char "..tbl[1].." is not in list")
				 return
			end
			self:Print(string.format("Deleted buddy name: %s => %s", tbl[1], BuddyName.db.realm.char[tbl[1]]));
			BuddyName.db.realm.char[tbl[1]]=nil
			return
	 end

	 -- list chars from database
	 if (command=="list") then
			--self:Print("This function is not yet available")

			self:Print("Buddy list:");

			-- TODO: Update block when switching to DB v2
			for p,v in pairs(BuddyName.db.realm.char) do
				 local char = p;
				 local buddy = v;
				 -- local buddy = BuddyName.db.realm.char[p].buddy -- DB v2
				 if (buddy and buddy~="") then
						self:Print(string.format("%s => %s", char, buddy));
				 end
			end

			-- TODO:
			-- Distinguish between all, guild, non-guild
			-- => Additional command arguments
			return
	 end

	 self:Print("Unknown command");

-- debug
--	 for v = 1,(#tbl) do
--			self:Print(ChatFrame1, tbl[v]);
--	 end
end

--[[ Executed only once on startup ]]
function BuddyName:OnInitialize()
	 --self:Print("OnInitialize called"); -- debug

	 -- Register Database, check entries
	 self.db = LibStub("AceDB-3.0"):New("BuddyName_DB")
	 if (self.db.realm.version==nil) then -- if no version number, then either fresh db or very old. In each case, reset realm DB.
			self.db.realm.version=BN_DB_Version -- set current version number
			self.db.realm.char = {} -- set empty char table
	 elseif (self.db.realm.version < BN_DB_Version) then -- update old database
			self:UpdateDB();
	 end

	 -- Register Slash commands
	 for p,v in pairs(slashCommands) do
			if (v == true) then
				 self:RegisterChatCommand(p, "SlashProcessor")
			end
	 end
end


--[[ Executed each time the addon is switched on ]]
function BuddyName:OnEnable()
	 --self:Print("OnEnable called"); -- debug

	 if (BN_Active) then
			return -- we're already active
	 else
			BN_Active = true;
	 end

	 -- activate chat filters
	 for p,v in pairs(chatEvents) do
			if (v == true) then
				 ChatFrame_AddMessageEventFilter(p, ChatFilter)
			end
	 end
end


--[[ Executed each time the addon is switched off ]]
function BuddyName:OnDisable()
	 --self:Print("OnDisable called");

	 if (not BN_Active) then
			return -- we're already inactive
	 else
			BN_Active = false;
	 end

	 -- remove chat filters
	 for p,v in pairs(chatEvents) do
			if (v == true) then
				 ChatFrame_RemoveMessageEventFilter(p, ChatFilter)
			end
	 end

end
